@@ -41,6 +41,8 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <sys/ioctl.h>
+#include <pwd.h>
+#include <grp.h>
 #if defined(__solaris__)
 #include <sys/filio.h>
 #endif
@@ -135,8 +137,8 @@ extern int	errno;
 */
 #define GKRELLMD_VERSION_MAJOR   2
 #define GKRELLMD_VERSION_MINOR   1
-#define GKRELLMD_VERSION_REV     5
-#define GKRELLMD_EXTRAVERSION    ""
+#define GKRELLMD_VERSION_REV     7
+#define GKRELLMD_EXTRAVERSION    "a"
 
 #define GKRELLMD_CONFIG     "gkrellmd.conf"
 
