@@ -47,6 +47,14 @@ static GList	*allow_host_list;
 
 static gboolean	detach_flag;
 
+#if !defined(WIN32)
+struct
+	{
+	uid_t	uid;
+	uid_t	gid;
+	}
+	drop_privs = { 0, 0 };
+#endif
 
 static void
 cb_sigterm(gint sig)
@@ -288,6 +296,26 @@ parse_config(gchar *config, gchar *arg)
 		_GK.fs_interval = atoi(arg);
 	else if (!strcmp(config, "nfs-interval"))
 		_GK.nfs_interval = atoi(arg);
+#if !defined(WIN32)
+	else if (!strcmp(config, "user") || !strcmp(config, "U"))
+		{
+		struct passwd *tmp;
+
+		if ((tmp = getpwnam(arg)) != (struct passwd*) 0)
+			drop_privs.uid = tmp->pw_uid;
+		else
+			return -1;
+		}
+	else if (!strcmp(config, "group") || !strcmp(config, "G"))
+		{
+		struct group *tmp;
+
+		if ((tmp = getgrnam(arg)) != (struct group*) 0)
+			drop_privs.gid = tmp->gr_gid;
+		else
+			return -1;
+		}
+#endif
 	else
 		return -1;
 	return 1;
@@ -361,6 +389,8 @@ usage(void)
 	printf(_("   -c, --clear-hosts      Clears the current list of allowed hosts.\n"));
 #if !defined(WIN32)
 	printf(_("   -d, --detach           Run in background and detach from terminal.\n"));
+	printf(_("   -U, --user username    Change to this username after startup.\n"));
+	printf(_("   -G, --group groupname  Change to this group after startup.\n"));
 #endif
 	printf(_("   -V, --verbose\n"));
 	printf(_("   -v, --version\n"));
@@ -532,7 +562,11 @@ detach_from_terminal(void)
 		return;
 
 #if defined(HAVE_DAEMON)
-	daemon(0, 0);
+	if (daemon(0, 0))
+		{
+		fprintf(stderr, "gkrellmd detach failed: %s\n", strerror(errno));
+		exit(1);
+		}
 #else
 	i = fork();
 	if (i > 0)
@@ -563,6 +597,23 @@ detach_from_terminal(void)
 #endif
 	}
 
+
+
+static void
+drop_privileges(void)
+	{
+#if !defined(WIN32)
+	if (drop_privs.gid > (uid_t)0)
+		{
+		(void) setgroups((size_t)0, (gid_t*)0);
+		(void) setgid(drop_privs.gid);
+		}
+	if (drop_privs.uid > (uid_t)0)
+		(void) setuid(drop_privs.uid);
+#endif
+	}
+
+
 #if defined(WIN32) && defined(_WINDOWS)
 int APIENTRY WinMain(HINSTANCE	hInstance,
 					HINSTANCE	hPrevInstance,
@@ -621,6 +672,7 @@ main(gint argc, gchar **argv)
 		}
 
 	gkrellm_sys_main_init();
+	drop_privileges();
 	g_thread_init(NULL);
 
 	_GK.start_time = time(0);
