@@ -1972,9 +1972,17 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 	{
 	GList			*list;
 	GkrellmdMonitor	*mon;
+	struct lconv	*lc;
 	gchar			buf[32], *s, *name;
 
 	send_to_client(client->fd, "<gkrellmd_setup>\n");
+
+	lc = localeconv();
+	sprintf(buf, "%c\n", *lc->decimal_point);
+	s = g_strconcat("<decimal_point>\n", buf, "\n", NULL);
+	send_to_client(client->fd, s);
+	g_free(s);
+
 	for (list = gkrellmd_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmdMonitor *) list->data;
