@@ -75,6 +75,22 @@ static gint		input_id;
 static gchar	server_buf[4097];
 static gint		buf_index;
 
+static gchar	locale_decimal_point;
+static gchar	server_decimal_point = '.';
+static gboolean	need_locale_fix = FALSE;
+
+
+  /* There can be a locale decimal point mismatch with the server.
+  */
+static void
+locale_fix(gchar *buf)
+	{
+	gchar   *s;
+
+	for (s = buf; *s; ++s)
+		if (*s == server_decimal_point)
+			*s = locale_decimal_point;
+	}
 
 
 /* ================================================================= */
@@ -198,6 +214,8 @@ static struct
 static void
 client_proc_line_from_server(gchar *line)
 	{
+	if (need_locale_fix)
+		locale_fix(line);
 	sscanf(line, "%d %d %lu %f %d", &proc.n_processes, &proc.n_running,
 			&proc.n_forks, &proc.load, &proc.n_users);
 	}
@@ -549,6 +567,12 @@ read_tcp_data(void)
 		}
 	}
 
+static void
+client_inet_reset(void)
+	{
+	free_glist_and_data(&inet_list);
+	}
+
 static void
 client_sys_inet_init(void)
 	{
@@ -884,6 +908,8 @@ client_sensors_line_from_server(gchar *line)
 	Sensor	s, *sensor;
 	gchar	basename[128];
 
+	if (need_locale_fix)
+		locale_fix(line);
 	sscanf(line, "%d \"%128[^\"]\" %d %d %d %f",
 			&s.type, basename, &s.id, &s.iodev, &s.inter, &s.raw_value);
 	for (list = sensors_list; list; list = list->next)
@@ -963,8 +989,8 @@ get_voltage(gchar *path, gint id, gint iodev, gint inter, gfloat *value)
 static void
 client_sys_sensors_init(void)
 	{
-	GList	*list;
-	Sensor	*s;
+	GList			*list;
+	Sensor			*s;
 
 	if (!sensors_list)
 		return;
@@ -990,6 +1016,9 @@ client_sensors_setup(gchar *line)
 	Sensor	*s;
 	gchar	basename[128], vref[32], default_label[32];
 
+	if (need_locale_fix)
+		locale_fix(line);
+
 	s = g_new0(Sensor, 1);
 	if (sscanf(line,
 			"%d \"%128[^\"]\" %d %d %d %f %f \"%32[^\"]\" \"%32[^\"]\"",
@@ -1113,6 +1142,25 @@ client_server_error(gchar *line)
 	exit(0);
 	}
 
+
+static void
+locale_sync(void)
+	{
+	struct lconv	*lc;
+
+	lc = localeconv();
+	locale_decimal_point = *lc->decimal_point;
+	if (locale_decimal_point != server_decimal_point)
+		need_locale_fix = TRUE;
+	}
+
+static void
+client_server_decimal_point(gchar *line)
+	{
+	sscanf(line, "%c", &server_decimal_point);
+	locale_sync();
+	}
+
 /* ================================================================= */
 KeyTable	setup_table[] =
 	{
@@ -1128,6 +1176,7 @@ KeyTable	setup_table[] =
 	{"<battery_setup>",	client_battery_setup },
 	{"<time>",			client_time_line_from_server},
 	{"<monitors>",		client_monitor_setup },
+	{"<decimal_point>",	client_server_decimal_point },
 	{"<error>",			client_server_error },
 	};
 
@@ -1227,9 +1276,13 @@ process_server_line(KeyTable *table, gint table_size, gchar *line)
 static void
 read_server_setup(gint fd)
 	{
-	gchar	buf[256];
-	gint	table_size;
+	gchar			buf[256];
+	gint			table_size;
 
+	/* Pre 2.1.6 gkrellmd does not send <decimal_point>, so put a fallback
+	|  locale_sync() here.
+	*/
+	locale_sync();
 	table_size = sizeof(setup_table) / sizeof(KeyTable);
 	while (1)
 		{
@@ -1239,6 +1292,11 @@ read_server_setup(gint fd)
 		process_server_line(&setup_table[0], table_size, buf);
 		}
 
+	/* Reset any data that is not cumulative.  gkrellmd sends .clear tags
+	|  for fstab and mounts, but inet does not.  So fix it here.
+	*/
+	client_inet_reset();
+
 	/* Read the initial update data
 	*/
 	table_size = sizeof(update_table) / sizeof(KeyTable);
