@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2002 Bill Wilson
+|  Copyright (C) 1999-2003 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -2227,6 +2227,8 @@ struct	_config	user_config[] =
 	{"chart_width",		&_GK.chart_width,		NULL,	0  },
 
 	{"update_HZ",		&_GK.update_HZ,			NULL,	0  },
+	{"allow_multiple_instances", &_GK.allow_multiple_instances,	NULL,	0  },
+	{"set_wm_state_hack", &_GK.set_wm_state_hack,	NULL,	0  }
 	};
 
   /* The user_config is read twice.  Early to load _GK config values and then
