@@ -394,7 +394,7 @@ update_cpu(void)
 	GkrellmChart	*cp;
 	GkrellmPanel	*p;
 	GkrellmKrell	*krell;
-	gulong			total, krell_value;
+	gulong			total, krell_value, full_scale;
 
 	_GK.cpu_sys_activity = 0;
 	if (composite_cpu)
@@ -429,8 +429,10 @@ update_cpu(void)
 			draw_sensor_decals(cpu);
 
 		krell = KRELL(cp->panel);
-		gkrellm_set_krell_full_scale(krell,
-					(gint) (total - cpu->previous_total), 10);
+		full_scale = total - cpu->previous_total;
+		if (full_scale > 0)		/* Can be 0 for data from gkrellmd */
+			gkrellm_set_krell_full_scale(krell, (gint) full_scale, 10);
+
 		if (cpu->previous_total > 0)
 			{
 			krell_value = cpu->sys + cpu->user;
@@ -611,7 +613,7 @@ cb_cpu_fan_alert_trigger(GkrellmAlert *alert, CpuMon *cpu)
   |   |     1           1     ||     1       1          1       |
   |   |----------------------------------------------------------
   */
-static gint
+static gboolean
 adjust_sensors_display(CpuMon *cpu, gint force)
 	{
 	GkrellmPanel	*p;
@@ -677,9 +679,8 @@ adjust_sensors_display(CpuMon *cpu, gint force)
 		gkrellm_draw_panel_label(p);
 		draw_sensor_decals(cpu);
 		gkrellm_draw_panel_layers(p);
-		return TRUE;
 		}
-	return FALSE;
+	return TRUE;
 	}
 
 static gint
@@ -954,34 +955,40 @@ load_cpu_config(gchar *arg)
 		}
 	}
 
-static void
+static gboolean
 fix_panel(CpuMon *cpu)
 	{
-	if (adjust_sensors_display(cpu, FALSE) && cpu->launch.button)
+	gboolean	result;
+
+	if ((result = adjust_sensors_display(cpu, FALSE)) && cpu->launch.button)
 		{
 		gkrellm_destroy_button(cpu->launch.button);
 		cpu->launch.button = 
 			gkrellm_put_label_in_panel_button(cpu->chart->panel,
 				gkrellm_launch_button_cb, &cpu->launch, cpu->launch.pad);
 		}
+	return result;
 	}
 
-void
+gboolean
 gkrellm_cpu_set_sensor(gpointer sr, gint type, gint n)
 	{
 	CpuMon	*cpu;
 
 	if (n_smp_cpus > 0)
 		++n;
-	if ((cpu = (CpuMon *) g_list_nth_data(cpu_mon_list, n)) == NULL)
-		return;
+	if (   (cpu = (CpuMon *) g_list_nth_data(cpu_mon_list, n)) == NULL
+		|| !cpu->enabled
+	   )
+		return FALSE;
+
 	if (type == SENSOR_TEMPERATURE)
 		cpu->sensor_temp = sr;
 	else if (type == SENSOR_FAN)
 		cpu->sensor_fan = sr;
 	else
-		return;
-	fix_panel(cpu);
+		return FALSE;
+	return fix_panel(cpu);
 	}
 
 static void
