@@ -387,11 +387,11 @@ GtkItemFactory *create_item_factory_popup(void);
 void	gkrellm_apply_hostname_config(void);
 
 gchar	*gkrellm_proc_get_sensor_panel_label(void);
-void	gkrellm_proc_set_sensor(gpointer sr, gint type);
+gboolean gkrellm_proc_set_sensor(gpointer sr, gint type);
 void	gkrellm_proc_draw_sensors(gpointer sr);
 
 gchar	*gkrellm_cpu_get_sensor_panel_label(gint n);
-void	gkrellm_cpu_set_sensor(gpointer sr, gint type, gint n);
+gboolean gkrellm_cpu_set_sensor(gpointer sr, gint type, gint n);
 void	gkrellm_cpu_draw_sensors(gpointer sr);
 
 /* utils.c */
