@@ -1499,8 +1499,16 @@ cb_size_allocate(GtkWidget *w, GtkAllocation *size, gpointer data)
 static gboolean
 cb_map_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 	{
+	/* Various window managers have bugs where the _NET_WM_STATE property set
+	|  prior to window mapping is not recognized.  As a temporary hack, users
+	|  can edit ~/.gkrellm2/user_config and set the set_wm_state_hack to 1 so
+	|  after mapping send extra _NET_WM_STATE changed messages in an attempt
+	|  to get these window managers to work.
+	*/
 	if (_GK.set_wm_state_hack)	/* Temporary option */
 		{
+		if (_GK.sticky_state)
+			gtk_window_stick(GTK_WINDOW(top_window));
 		if (_GK.state_skip_pager)
 			gkrellm_winop_state_skip_pager(TRUE);
 		if (_GK.state_skip_taskbar)
@@ -1977,7 +1985,9 @@ main(gint argc, gchar **argv)
 
 	load_user_config(FALSE);		
 	decorated = (_GK.command_line_decorated || _GK.decorated);
-	if (_GK.command_line_plugin || _GK.command_line_theme)
+	if (   _GK.command_line_plugin || _GK.command_line_theme
+		|| _GK.debug_level > 0 || _GK.debug > 0
+	   )
 		_GK.allow_multiple_instances_real = TRUE;
 	_GK.allow_multiple_instances_real |= _GK.allow_multiple_instances;
 
