@@ -34,7 +34,8 @@
 typedef struct
 	{
 	gchar			*name;
-	GtkWidget		*vbox;			/* Handle for hidding */
+	GtkWidget		*vbox,
+					*parent_vbox;
 	GtkWidget		*enable_button,
 					*force_button,
 					*label_entry;
@@ -108,14 +109,38 @@ static gboolean	net_use_routed;
 static gboolean	net_config_use_routed;
 
 
+
+static gint
+strcmp_net_name(NetMon *net1, NetMon *net2)
+	{
+	gchar	*s;
+	gint	n, n1, n2, len;
+
+	for (s = net1->name; *s; ++s)
+		if (isdigit(*s))
+			break;
+	if (!*s)
+		return strcmp(net1->name, net2->name);
+	n1 = atoi(s);
+	len = s - net1->name;
+	n = strncmp(net1->name, net2->name, len);
+	if (n == 0)
+		{
+		n2 = atoi(net2->name + len);
+		if (n1 < n2)
+			n = -1;
+		else if (n1 > n2)
+			n = 1;
+		}
+	return n;
+	}
+
 static NetMon *
 new_net(gchar *name)
 	{
 	NetMon	*net;
 
 	net = g_new0(NetMon, 1);
-	net_mon_list = g_list_append(net_mon_list, net);
-
 	net->name = g_strdup(name);
 	if (strcmp(name, "lo"))
 		net->enabled = TRUE;		/* All except lo default to enabled */
@@ -123,6 +148,10 @@ new_net(gchar *name)
 	net->label = g_strdup("");
 	net->launch.command = g_strdup("");
 	net->launch.tooltip_comment = g_strdup("");
+
+	net_mon_list = g_list_insert_sorted(net_mon_list, net,
+				(GCompareFunc) strcmp_net_name);
+
 	return net;
 	}
 
@@ -819,6 +848,7 @@ destroy_chart(NetMon *net)
 		gtk_widget_destroy(net->vbox);
 		net->chart = NULL;
 		net->vbox = NULL;
+		net->parent_vbox = NULL;
 		}
 	net->locked = FALSE;
 	}
@@ -1045,6 +1075,28 @@ cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
 	return FALSE;
 	}
 
+  /* Make sure net charts appear in same order as the sorted net_mon_list
+  */
+static void
+net_chart_reorder(NetMon *new_net)
+	{
+	NetMon		*net;
+	GList		*list;
+	gint		i;
+
+	for (i = 0, list = net_mon_list; list; list = list->next)
+		{
+		net = (NetMon *) list->data;
+		if (net->parent_vbox != new_net->parent_vbox)
+			continue;
+		if (net == new_net)
+			break;
+		++i;
+		}
+	if (!list || !list->next)
+		return;
+	gtk_box_reorder_child(GTK_BOX(new_net->parent_vbox), new_net->vbox, i);
+	}
 
 static void
 create_net_monitor(GtkWidget *vbox, NetMon *net, gint first_create)
@@ -1055,12 +1107,14 @@ create_net_monitor(GtkWidget *vbox, NetMon *net, gint first_create)
 
 	if (first_create)
 		{
+		net->parent_vbox = vbox;
 		net->vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(vbox), net->vbox);
+		gtk_box_pack_start(GTK_BOX(vbox), net->vbox, FALSE, FALSE, 0);
 
 		net->chart = gkrellm_chart_new0();
 		net->chart->panel = gkrellm_panel_new0();
 		net->chart->panel->textstyle = gkrellm_textstyle_new0();
+		net_chart_reorder(net);
 		}
 	cp = net->chart;
 	p = cp->panel;
