@@ -411,7 +411,7 @@ cb_mb_fan_alert_trigger(GkrellmAlert *alert, ProcMon *proc)
   |   |     1           1     ||     1       1          1       |
   |   |----------------------------------------------------------
   */
-static gint
+static gboolean
 adjust_sensors_display(gint force)
 	{
 	GkrellmPanel	*p;
@@ -477,9 +477,8 @@ adjust_sensors_display(gint force)
 		gkrellm_draw_panel_label(p);
 		draw_sensor_decals();
 		gkrellm_draw_panel_layers(p);
-		return TRUE;
 		}
-	return FALSE;
+	return TRUE;
 	}
 
 static gint
@@ -503,7 +502,6 @@ create_proc_monitor(GtkWidget *vbox, gint first_create)
 		{
 		proc.chart = gkrellm_chart_new0();
 		proc.chart->panel = gkrellm_panel_new0();
-		proc.panel_label = g_strdup(_("Proc"));
 		}
 	cp = proc.chart;
 	p = cp->panel;
@@ -625,21 +623,24 @@ static GtkWidget	*proc_launch_entry,
 static GtkWidget	*text_format_combo;
 
 
-static void
+static gboolean
 fix_panel(void)
 	{
+	gboolean	result;
+
 	if (!proc.enabled)
-		return;
-	if (adjust_sensors_display(FALSE) && proc_launch.button)
+		return FALSE;
+	if ((result = adjust_sensors_display(FALSE)) && proc_launch.button)
 		{
 		gkrellm_destroy_button(proc_launch.button);
 		proc_launch.button = 
 			gkrellm_put_label_in_panel_button(proc.chart->panel,
 				gkrellm_launch_button_cb, &proc_launch, proc_launch.pad);
 		}
+	return result;
 	}
 
-void
+gboolean
 gkrellm_proc_set_sensor(gpointer sr, gint type)
 	{
 	if (type == SENSOR_TEMPERATURE)
@@ -647,8 +648,8 @@ gkrellm_proc_set_sensor(gpointer sr, gint type)
 	else if (type == SENSOR_FAN)
 		proc.sensor_fan = sr;
 	else
-		return;
-	fix_panel();
+		return FALSE;
+	return fix_panel();
 	}
 
 static void
@@ -824,6 +825,7 @@ init_proc_monitor(void)
 	GkrellmChartconfig	*cf;
 
 	monitor_proc.name = _(monitor_proc.name);
+	proc.panel_label = g_strdup(_("Proc"));
 	proc.enabled = TRUE;
 	proc.extra_info = TRUE;
 	style_id = gkrellm_add_chart_style(&monitor_proc, PROC_STYLE_NAME);
