@@ -596,6 +596,7 @@ static void
 add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 	{
 	SensorMon		*smon;
+	gboolean		set_loc;
 	static void		cb_alert_config(GkrellmAlert *ap, Sensor *sr);
 
 	sr->smon = NULL;
@@ -605,11 +606,13 @@ add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 	if (sr->location != SENSOR_PANEL_LOCATION)
 		{
 		if (sr->location == PROC_PANEL_LOCATION)
-			gkrellm_proc_set_sensor(sr, sr->type);
+			set_loc = gkrellm_proc_set_sensor(sr, sr->type);
 		else
-			gkrellm_cpu_set_sensor(sr, sr->type,
+			set_loc = gkrellm_cpu_set_sensor(sr, sr->type,
 						sr->location - CPU_PANEL_LOCATION);
-		return;
+		if (set_loc)
+			return;
+		sr->location = SENSOR_PANEL_LOCATION;
 		}
 	smon = g_new0(SensorMon, 1);
 	smon->sensor = sr;
@@ -2039,6 +2042,10 @@ cb_location_menu(GtkOptionMenu *om, gpointer data)
 				s->location = SENSOR_PANEL_LOCATION;	/* is being replaced */
 			}
 	rebuild_sensors_monitor(DO_TEMP, DO_FAN, FALSE);
+
+	if (sr->location != location)	/* location failed */
+		gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),
+					SENSOR_PANEL_LOCATION);
 	}
 
 static GtkWidget *
