@@ -918,6 +918,10 @@ static struct mibtab mibs[] = {
 #define	PROC_MEMINFO_FILE	"/compat/linux/proc/meminfo"
 #endif
 
+#ifndef VM_TOTAL
+#define VM_TOTAL	VM_METER
+#endif
+
 static guint64	swapin,
 		swapout,
 		swap_total,
@@ -935,7 +939,7 @@ gkrellm_sys_mem_read_data(void)
 	guint64		total, used, x_used, free, shared, buffers, cached;
 	struct vmtotal	vmt;
 	size_t		length_vmt = sizeof(vmt);
-	static int	oid_vmt[] = { CTL_VM, VM_METER };
+	static int	oid_vmt[] = { CTL_VM, VM_TOTAL };
 #if __FreeBSD_version >= 410000
 	gint		i;
 	FILE		*f;
