@@ -21,9 +21,12 @@
 #include <limits.h>
 #include <errno.h>
 #include <locale.h>
+#include <sys/types.h>
+#include <dirent.h>
 
 static gboolean		need_locale_fix,
-					have_partition_stats;
+					have_partition_stats,
+					have_sysfs_stats;
 static gchar		locale_decimal_point;
 
 void
@@ -257,6 +260,9 @@ gkrellm_sys_cpu_init(void)
 #if !defined(COMPAQ_CISS_MAJOR)
 #define COMPAQ_CISS_MAJOR	104
 #endif
+#if !defined(LVM_BLK_MAJOR)
+#define LVM_BLK_MAJOR 58
+#endif
 
 
 struct _disk_name_map
@@ -267,8 +273,7 @@ struct _disk_name_map
 	gchar	suffix_base;
 	};
 
-  /* Disk charts will appear in GKrellM in the same order as this table
-  |  if using /proc/stat.  /proc/partitions disks appear in the order there.
+  /* Disk charts will appear in GKrellM in the same order as this table.
   */
 static struct _disk_name_map
 	disk_name_map[] =
@@ -283,7 +288,7 @@ static struct _disk_name_map
 	{"hd",	IDE7_MAJOR,					64,	'o' },	/* 89: hdo, hdp */
 	{"hd",	IDE8_MAJOR,					64,	'q' },	/* 90: hdq, hdr */
 	{"hd",	IDE9_MAJOR,					64,	's' },	/* 91: hds, hdt */
-	{"sd",	SCSI_DISK0_MAJOR,			0,	'a' },	/* 8:  sda-sdh */
+	{"sd",	SCSI_DISK0_MAJOR,			16,	'a' },	/* 8:  sda-sdh */
 	{"sg",	SCSI_GENERIC_MAJOR,			0,	'0' },	/* 21: sg0-sg16 */
 	{"scd",	SCSI_CDROM_MAJOR,			0,	'0' },	/* 11: scd0-scd16 */
 	{"md",	MD_MAJOR,					0,	'0' },	/* 9:  md0-md3 */
@@ -375,23 +380,38 @@ gkrellm_sys_disk_order_from_name(gchar *name)
 	return i;
 	}
 
-
-gboolean
-expanded_partition_name_assign(gchar *name, gint major, gint minor,
-			gulong rsect, gulong wsect)
+  /* Given a /proc/partitions line disk name in "partition", make "disk" have
+  |  the whole disk name (eg hda) and "partition" have the partition (eg hda1)
+  |  or NULL if not a partition.  For simple names, "disk" is expected to
+  |  initially have the whole disk name from the previous call (or NULL if
+  |  this is the first call per /proc/partitions parse).
+  */
+static gboolean
+get_proc_partition_name(gint major, gint minor, gchar *disk, gchar *partition)
 	{
 	struct _disk_name_map	*dm	= NULL;
 	gint					i, unit = 0;
-	gchar					*p;
-	gchar 					disk[32], part[32];
-
-	for (p = name; *p; ++p)
-		if (*p == '/')		/* name like ide/host0/bus0/target0/lun0/part1 */
+	gchar					*p, *d;
+		
+	for (p = partition; *p; ++p)
+		if (*p == '/')
 			break;
 	if (!*p)
-		return FALSE;		/* Will use /proc/partition name */
+		{	/* Have a simple /proc/partition name like hda, hda1, sda, ... */
+		d = disk;
+		p = partition;
+		while (*d && *p && *d++ == *p++)
+			;
+		if (d == disk || *d || *p < '0' || *p > '9')
+			{
+			strcpy(disk, partition);
+			partition[0] = '\0';
+			}
+		return TRUE;
+		}
 
-	/* Construct the name based on major, minor numbers
+	/* Have a devfs name like ide/host0/bus0/target0/lun0/part1, so construct
+	|  a name based on major, minor numbers and the disk_name_map[].
 	*/
 	for (i = 0; i < sizeof(disk_name_map) / sizeof(struct _disk_name_map); ++i)
 		{
@@ -405,13 +425,9 @@ expanded_partition_name_assign(gchar *name, gint major, gint minor,
 			}
 		sprintf(disk, "%s%c", dm->name, dm->suffix_base + unit);
 		if (minor > 0)
-			{
-			sprintf(part, "%s%d", disk, minor);
-			gkrellm_disk_subdisk_assign_data_by_name(part, disk,
-							512 * rsect, 512 * wsect);
-			}
+			sprintf(partition, "%s%d", disk, minor);
 		else
-			gkrellm_disk_assign_data_by_name(disk, 512 * rsect, 512 * wsect);
+			partition[0] = '\0';
 		return TRUE;
 		}
 	return FALSE;
@@ -422,58 +438,71 @@ expanded_partition_name_assign(gchar *name, gint major, gint minor,
 void
 gkrellm_sys_disk_read_data(void)
 	{
-	FILE	*f;
-	gchar	buf[1024], partition[64], disk[64];
-	gchar	*d, *p;
-	gint	major, minor, n;
-	gulong	sectors, rd_sectors, wr_sectors;
+	FILE		*f, *sf;
+	gchar		buf[1024], part[64], disk[64], sysfspath[128];
+	gint		major, minor, n;
+	gulong		sectors, rd, wr;
 
-	/* One routine reads cpu, disk, and swap data.  All three monitors will
-	| call it, but only the first call per timer tick will do the work.
-	*/
-	linux_read_proc_stat();		/* handles floppys and CDroms */
-	if (   !have_partition_stats
-		|| (f = fopen(PROC_PARTITIONS_FILE, "r")) == NULL
-	   )
+	if (!have_sysfs_stats)
+		linux_read_proc_stat();		/* handles floppys and CDroms */
+
+	if (!have_partition_stats && !have_sysfs_stats)
 		return;
-	fgets(buf, sizeof(buf), f);		/* waste header */
-	disk[0] = '\0';
-	while ((fgets(buf, sizeof(buf), f)) != NULL)
-		{
-		/* major minor  #blocks  name
-		|  rio rmerge rsect ruse wio wmerge wsect wuse
-		|  running use aveq
-		*/
-		n = sscanf(buf, "%d %d %lu %64s %*d %*d %lu %*d %*d %*d %lu",
-				&major, &minor, &sectors, partition, &rd_sectors, &wr_sectors);
-		if (n < 6 || sectors <= 1)
-			continue;
 
-		/* It would be nice if /proc/partitions had consistent simple disk
-		|  names I could directly use, but there can be for example an expanded
-		|  ide/host0/bus0/target0/lun0/part1 instead of hda1.  I'll use the
-		|  the /proc/partitions name if I can.
-		*/
-		if (!expanded_partition_name_assign(partition, major, minor,
-					rd_sectors, wr_sectors)
-		   )
+	if ((f = fopen(PROC_PARTITIONS_FILE, "r")) != NULL)
+		{
+		fgets(buf, sizeof(buf), f); /* nuke header */
+		fgets(buf, sizeof(buf), f); 
+		disk[0] = '\0';
+		part[0] = '\0';
+		
+		while ((fgets(buf, sizeof(buf), f)) != NULL)
 			{
-			d = disk;
-			p = partition;
-			while (*d && *p && *d++ == *p++)
-				;
-			if (!*d && *p >= '0' && *p <= '9')
-				gkrellm_disk_subdisk_assign_data_by_name(partition, disk,
-							512 * rd_sectors, 512 * wr_sectors);
-			else
+			if (have_partition_stats)
 				{
-				strcpy(disk, partition);
-				gkrellm_disk_assign_data_by_name(disk,
-							512 * rd_sectors, 512 * wr_sectors);
+				/* major minor  #blocks  name
+				|  rio rmerge rsect ruse wio wmerge wsect wuse running use aveq
+				*/
+				n = sscanf(buf, "%d %d %lu %64s %*d %*d %lu %*d %*d %*d %lu",
+							&major, &minor, &sectors, part, &rd, &wr);
+				if (   n < 6 || sectors <= 1 || major == LVM_BLK_MAJOR
+					|| !get_proc_partition_name(major, minor, disk, part)
+				   )
+					continue;
 				}
+			if (have_sysfs_stats)
+				{
+				n = sscanf(buf, "%d %d %lu %64s",
+							&major, &minor, &sectors, part);
+				if (   n < 4 || sectors <= 1 || major == LVM_BLK_MAJOR
+					|| !get_proc_partition_name(major, minor, disk, part)
+				   )
+					continue;
+				if (part[0] == '\0')
+					sprintf(sysfspath, "/sys/block/%s/stat", disk);
+				else
+					sprintf(sysfspath, "/sys/block/%s/%s/stat", disk, part);
+				if ((sf = fopen(sysfspath, "r")) != NULL)
+					{
+					fgets(buf, sizeof(buf), sf);
+					fclose(sf);
+					if (part[0] == '\0')
+						n = sscanf(buf,"%*d %*d %lu %*d %*d %*d %lu",&rd, &wr);
+					else
+						n = sscanf(buf,"%*d %lu %*d %lu", &rd, &wr);
+					if (n < 2)
+						continue;
+					}
+				}
+			if (part[0] == '\0')
+				gkrellm_disk_assign_data_by_name(disk, 512 * rd, 512 * wr);
+			else
+				gkrellm_disk_subdisk_assign_data_by_name(part, disk,
+							512 * rd, 512 * wr);
 			}
+			fclose(f);
 		}
-	fclose(f);
+		return;
 	}
 
 gboolean
@@ -484,10 +513,24 @@ gkrellm_sys_disk_init(void)
 
 	if ((f = fopen(PROC_PARTITIONS_FILE, "r")) != NULL)
 		{
-		if (   fgets(buf, sizeof(buf), f)
-			&& strstr(buf, "rsect")
-		   )
-			have_partition_stats = TRUE;
+		if (fgets(buf, sizeof(buf), f))
+			{
+			if (strstr(buf, "rsect"))
+				have_partition_stats = TRUE;
+			else 
+				{
+				/* stats are in sysfs since 2.5.47, but it may not be mounted
+				*/
+				fclose(f);
+				if ((f = fopen("/proc/mounts", "r")) != NULL)
+					while ((fgets(buf, sizeof(buf), f)) != NULL)
+						if (strstr(buf, "sysfs"))
+							{
+							have_sysfs_stats = TRUE;
+							break;
+							}
+				}
+			}
 		fclose(f);
 		}
 	return TRUE;
@@ -1071,6 +1114,7 @@ gkrellm_sys_fs_get_fstab_list(void)
 			|| !strcmp(type, "devpts")
 			|| !strcmp(type, "swap")
 			|| !strcmp(type, "proc")
+			|| !strcmp(type, "sysfs")
 			|| !strcmp(type, "usbdevfs")
 			|| !strcmp(type, "ignore")
 		   )
@@ -1113,6 +1157,7 @@ gkrellm_sys_fs_get_mounts_list(void)
 		if (   !strcmp(type, "devpts")
 			|| !strcmp(type, "proc")
 			|| !strcmp(type, "usbdevfs")
+			|| !strcmp(type, "sysfs")
 		   )
 			continue;
 		/* Strip trailing / from the directory.
@@ -1244,19 +1289,36 @@ setup_ac_adapter(gchar **state, gchar *ac)
 static void
 acpi_setup(void)
 	{
-	setup_acpi_battery("BAT0");
-	setup_acpi_battery("BAT1");
-	setup_acpi_battery("BAT2");
-	setup_acpi_battery("BAT3");
+	DIR				*d;
+	struct dirent	*de;
+
+	if ((d = opendir(ACPI_BATTERY_DIR)) == NULL)
+		return;
+	
+	while ((de = readdir(d)) != NULL)
+		{
+		if (   strcmp(de->d_name, ".")
+			&& strcmp(de->d_name, "..")
+		   )
+			setup_acpi_battery(de->d_name);
+		}
+	closedir(d);
 
 	if (!acpi_battery_list)
 		return;
-
-	if (   !setup_ac_adapter(&acpi_ac_state_file, "AC")
-		&& !setup_ac_adapter(&acpi_ac_state_file, "ADP1")
-		&& !setup_ac_adapter(&acpi_ac_state_file, "ACAD")
-	   )
-		acpi_battery_list = NULL;	/* forget it */
+	
+	if ((d = opendir(ACPI_AC_ADAPTOR_DIR)) != NULL)
+		{
+		while ((de = readdir(d)) != NULL)
+			{
+			if (   strcmp(de->d_name, ".")
+				&& strcmp(de->d_name, "..")
+				&& setup_ac_adapter(&acpi_ac_state_file, de->d_name)
+			   )
+				break;
+			}
+		closedir(d);
+		}
 	}
 
 static gboolean
@@ -1298,19 +1360,23 @@ acpi_battery_data(BatteryFile *bf)
 	if (bf->full_cap == 0)
 		bf->full_cap = gkrellm_battery_full_cap_fallback();
 
-	if ((f = fopen(acpi_ac_state_file, "r")) == NULL)
-		return FALSE;
 	on_line = FALSE;
-	while (fgets(buf, sizeof(buf), f))
+
+	if (   acpi_ac_state_file
+		&& (f = fopen(acpi_ac_state_file, "r")) != NULL
+	   )
 		{
-		/*	state: {on-line|off-line}
-		*/
-		if (   sscanf (buf, "state: %s", s1) == 1
-			&& !strcmp(s1, "on-line")
-		   )
-			on_line = TRUE;
+		while (fgets(buf, sizeof(buf), f))
+			{
+			/*	state: {on-line|off-line}
+			*/
+			if (   sscanf (buf, "state: %s", s1) == 1
+				&& !strcmp(s1, "on-line")
+			   )
+				on_line = TRUE;
+			}
+		fclose(f);
 		}
-	fclose(f);
 
 	if ((f = fopen(bf->state, "r")) == NULL)
 		return FALSE;
@@ -1344,6 +1410,9 @@ acpi_battery_data(BatteryFile *bf)
 		}
 	fclose(f);
 
+	if (!acpi_ac_state_file && charging)	/* Assumption for buggy ACPI */
+		on_line = TRUE;
+
 	percent = cur_cap * 100 / bf->full_cap;
 	if (percent > 100)
 		{
@@ -1454,8 +1523,6 @@ gkrellm_sys_uptime_init(void)
 /* Sensor monitor interface */
 /* ------- Linux ------------------------------------------------------- */
 
-#include <dirent.h>
-
 #define	SENSORS_DIR	"/proc/sys/dev/sensors"
 
 typedef struct
@@ -1590,7 +1657,7 @@ gkrellm_sys_sensors_get_temperature(gchar *device_name, gint id,
 	FILE	*f;
 	gchar	buf[64];
 	gint	n;
-	gfloat	t[5];
+	gfloat	T, t[5];
 
 	if ((f = fopen(device_name, "r")) == NULL)
 		return FALSE;
@@ -1602,9 +1669,12 @@ gkrellm_sys_sensors_get_temperature(gchar *device_name, gint id,
 	n = sscanf(buf, "%f %f %f %f %f", &t[0], &t[1], &t[2], &t[3], &t[4]);
 	if (n < 1)
 		return FALSE;
+	T = t[n - 1];
+	if (T > 254.0)		/* Bogus read from BIOS if CHAR_MAX */
+		return FALSE;
 
 	if (temp)
-		*temp = t[n - 1];
+		*temp = T;
 	return TRUE;
 	}
 
@@ -1639,7 +1709,7 @@ gkrellm_sys_sensors_get_voltage(gchar *device_name, gint id,
 	{
 	FILE	*f;
 	gchar	buf[64];
-	gfloat	t[3];
+	gfloat	V, t[3];
 	gint	n;
 
 	if ((f = fopen(device_name, "r")) == NULL)
@@ -1652,9 +1722,12 @@ gkrellm_sys_sensors_get_voltage(gchar *device_name, gint id,
 	n = sscanf(buf, "%f %f %f", &t[0], &t[1], &t[2]);
 	if (n < 1)
 		return FALSE;
+	V = t[n - 1];
+	if (V > 254.0)
+		return FALSE;
 
 	if (volt)
-		*volt = t[n - 1];
+		*volt = V;
 	return TRUE;
 	}
 
