@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2002 Bill Wilson
+|  Copyright (C) 1999-2003 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -22,6 +22,10 @@
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
+#include <errno.h>
+#if !defined(F_TLOCK)
+#include <sys/file.h>
+#endif
 
 #include <gdk/gdkx.h>
 #include <X11/Xmd.h>
@@ -206,6 +210,24 @@ gkrellm_winop_state_below(gboolean state)
 	net_wm_state("_NET_WM_STATE_BELOW", state);
 	}
 
+static FILE		*f_lock;
+
+static gboolean
+_gkrellm_get_lock(void)
+	{
+	gchar	*lock_file;
+
+#if defined(F_TLOCK)
+	lock_file = gkrellm_make_config_file_name(gkrellm_homedir(),
+					GKRELLM_LOCK_FILE);
+	f_lock = fopen(lock_file, "w+");	/* buffering does not apply here */
+	g_free(lock_file);
+	if (!f_lock || lockf(fileno(f_lock), F_TLOCK, 0) != 0)
+		return FALSE;
+#endif
+	return TRUE;
+	}
+
 void
 gkrellm_winop_options(gint argc, gchar **argv)
 	{
@@ -214,6 +236,8 @@ gkrellm_winop_options(gint argc, gchar **argv)
 	Atom		atoms[3];
 	gint		n = 0;
 
+	if (!_GK.allow_multiple_instances_real && !_gkrellm_get_lock())
+		exit(0);
 	smc_connect(argc, argv);
 	display = GDK_WINDOW_XDISPLAY(gkrellm_get_top_window()->window);
 	window  = GDK_WINDOW_XWINDOW(gkrellm_get_top_window()->window);
